//
//  IndividualThreadView.m
//  Newsgroup
//
//  Created by Jim Kubicek on 2/15/11.
//  Copyright 2011 jimkubicek.com. All rights reserved.
//

#import "IndividualThreadView.h"
#import "NewPostViewController.h"
#import "FetchedResultsViewController+PrivateHeader.h"

#define POST_LINK_PREFIX @"ViewPost.aspx?PostID="

@interface IndividualThreadView()

- (void)pressAndHoldOnSubject:(UILongPressGestureRecognizer *)gesture;

@end

#pragma mark -
@implementation IndividualThreadView

#pragma mark Instance Methods

- (void)newPost:(id)sender {
    [super newPost:sender];

    // Get the selected text
    NSString *javascript = @"window.getSelection().toString()";
    NSString *selectedtext = [self.webView stringByEvaluatingJavaScriptFromString:javascript];
    if ([selectedtext length] == 0)
        selectedtext = nil;


    // Setup and push the new view controller
    NewPostViewController *newPostController = [[NewPostViewController alloc] initWithNibName:@"NewPostView" bundle:nil];
    newPostController.parentPostID = self.post.postID;
    newPostController.subject = self.post.subject;
    newPostController.quotedText = selectedtext;

    [self presentViewController:newPostController animated:YES completion:nil];
}

- (void)pressAndHoldOnSubject:(UILongPressGestureRecognizer *)gesture {
    if (gesture.state == UIGestureRecognizerStateBegan) {
        // Copy URL string to clipboard
        NSString *urlString = [NSString stringWithFormat:@"<a href=\"http://newsgroup.greenpride.com/ViewPost.aspx?PostID=%@\"></a>", self.post.postID];
        [UIPasteboard generalPasteboard].string = urlString;
        
        // Flash the subject
        NSTimeInterval flashLength = 0.1;
        [UIView animateWithDuration:flashLength animations:^(void) {
            self.subjectLabel.alpha = 0.0f;
        } completion:^(BOOL finished) {
            [UIView animateWithDuration:flashLength animations:^(void) {
                self.subjectLabel.alpha = 1.0f;
            } completion:^(BOOL finished1) {
                [UIView animateWithDuration:flashLength animations:^(void) {
                    self.subjectLabel.alpha = 0.0f;
                } completion:^(BOOL finished2) {
                    self.subjectLabel.alpha = 1.0f;
                }];
            }];
        }];
    }
}

#pragma mark View lifecycle

- (void)viewDidLoad {
    [super viewDidLoad];
    
    // Setup the headerview
    UINib *headerNib = [UINib nibWithNibName:@"HeaderView" bundle:nil];
    [headerNib instantiateWithOwner:self options:nil];
    self.tableView.tableHeaderView = self.headerView;
    self.authorLabel.text = self.post.posterName;
    self.subjectLabel.text = self.post.subject;
    self.postTimeLabel.text = self.post.displayDate;
    [self.webView loadHTMLString:[DataController addBodyToHTMLTemplate:self.post.body] baseURL:nil];
    
    // Set up the gesture recognizer on the subject
    UILongPressGestureRecognizer *pressGesture = [[UILongPressGestureRecognizer alloc] initWithTarget:self action:@selector(pressAndHoldOnSubject:)];
    [self.subjectLabel addGestureRecognizer:pressGesture];
    
    // Set the current post as read
    if ([self.post.isRead boolValue] == NO) {
        self.post.isRead = [NSNumber numberWithBool:YES];
        [APP_DELEGATE.dataController markPostAsRead:self.post.postID];
    }
    
    // Add the current post to the history table
    [APP_DELEGATE.dataController addPostToHistory:self.post];
    
    // Fetch all our threads
    NSFetchedResultsController *fetchedResults = [APP_DELEGATE.dataController postsWithParentID:self.post.postID];
    fetchedResults.delegate = self;
    
    NSError *error = nil;
    if (![fetchedResults performFetch:&error]) {
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }
    
    self.fetchedResultsController = fetchedResults;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
}


// Customize the appearance of table view cells.
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
    UITableViewCell *cell = [super tableView:tableView cellForRowAtIndexPath:indexPath];
    
    // Configure the cell.
    [self configureCell:cell withPost:[self.fetchedResultsController objectAtIndexPath:indexPath]];
    
    return cell;
}

#pragma mark UIWebViewDelegate methods

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
    
    NSString *urlDescription = [[request URL] description];
    NSRange testRange = [urlDescription rangeOfString:POST_LINK_PREFIX];
    
    if ([urlDescription isEqualToString:@"about:blank"]) {
        return YES;
    } else if (testRange.location != NSNotFound) {
        NSRange removeRange = NSMakeRange(0, testRange.location + testRange.length);
        NSString *postIDString = [urlDescription stringByReplacingCharactersInRange:removeRange withString:@""];
        NSInteger postID = [postIDString integerValue];
        
        if (postID != 0) {
            if ([APP_DELEGATE navigateToPostID:[NSNumber numberWithInteger:postID]]) return NO;
        }
    }
        
    [[UIApplication sharedApplication] openURL:[request URL]];
    return NO;
}

@end

